/*
Open Asset Import Library (ASSIMP)
----------------------------------------------------------------------

Copyright (c) 2006-2008, ASSIMP Development Team
All rights reserved.

Redistribution and use of this software in source and binary forms, 
with or without modification, are permitted provided that the 
following conditions are met:

* Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

* Redistributions in binary form must reproduce the above
  copyright notice, this list of conditions and the
  following disclaimer in the documentation and/or other
  materials provided with the distribution.

* Neither the name of the ASSIMP team, nor the names of its
  contributors may be used to endorse or promote products
  derived from this software without specific prior
  written permission of the ASSIMP Development Team.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------
*/
  
/*! @file Som.h
*  @brief Declaration of the Sword of Moonlight formats loader main class
*/
#ifndef AI_SOMLOADER_H_INC
#define AI_SOMLOADER_H_INC

#include "BaseImporter.h"

namespace Assimp{ 

class SomImporter : public BaseImporter 
{
	friend class Importer;

protected:
	/** Constructor to be privately used by Importer */
	SomImporter();

	/** Destructor, private as well */
	~SomImporter();

public:

	// -------------------------------------------------------------------
	/** Returns whether the class can handle the format of the given file. 
	* See BaseImporter::CanRead() for details.	*/
	bool CanRead( const std::string& pFile, IOSystem* pIOHandler,
		bool checkSig) const;


	// -------------------------------------------------------------------
	/** Called prior to ReadFile().
	* The function is a request to the importer to update its configuration
	* basing on the Importer's configuration property list.
	*/
	void SetupProperties(const Importer* pImp);

protected:


	// -------------------------------------------------------------------
	/** Called by Importer::GetExtensionList() for each loaded importer.
	 * See BaseImporter::GetExtensionList() for details
	 */
	void GetExtensionList(std::set<std::string>& extensions);

	// -------------------------------------------------------------------
	/** Imports the given file into the given scene structure. 
	* See BaseImporter::InternReadFile() for details
	*/
	void InternReadFile( const std::string& pFile, aiScene* pScene, 
		IOSystem* pIOHandler);
		  
protected:

	void CheckEof(const void* szPos)const;
	void CheckEof(const void* szPos, const char* szFile, unsigned int iLine)const;
							   
	bool DetectFormat_MDL(); void InternReadFile_MDL(const std::string&);
	bool DetectFormat_MDO(); void InternReadFile_MDO();
	bool DetectFormat_MSM(); void InternReadFile_MSM();
	bool DetectFormat_MHM(); void InternReadFile_MHM();

protected:

	//HACK: x2msm sets a SOM_NOT_POSTPROCESSING property
	//in order to retain non-triangle polygons.
	int x2msm, sp_gen_connectivity, modernizing;

	/** Buffer to hold the loaded file */
	unsigned char* mBuffer;

	/** Output I/O handler. used to load external lmp files */
	IOSystem* pIOHandler;

	/** Output scene to be filled */
	aiScene* pScene;

	/** Size of the input file in bytes */
	unsigned int iFileSize;
};

} // end namespace Assimp

#endif // ! AI_SOMHELPER_H_IN